@@ -2,7 +2,10 @@
  *
  * Copyright (c) 1998-99  Jonathan A. Buzzard (jonathan@buzzard.org.uk)
  *
- * $Log:$
+ * $Log: alarm.c,v $
+ * Revision 1.1  1999/12/04 13:40:07  jab
+ * Initial revision
+ *
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,19 +23,13 @@
  *
  */
 
-static char const rcsid[]="$Id:$";
+static char const rcsid[]="$Id: alarm.c,v 1.1 1999/12/04 13:40:07 jab Exp jab $";
 
 
 #include<stdio.h>
 #include<stdlib.h>
 #include<unistd.h>
-#include<syslog.h>
-#include<signal.h>
-#include<paths.h>
 #include<time.h>
-#include<pwd.h>
-#include<features.h>
-#include<sys/stat.h>
 #ifdef __GLIBC__
 #include<sys/perm.h>
 #endif
@@ -41,21 +38,20 @@ static char const rcsid[]="$Id:$";
 #include"parsetime.h"
 
 
-#undef VERSION
-#define VERSION "\
+#define VERSION_STRING "\
 Alarm version 1.0\n\
 Copyright (c) 1998-99 Jonathan A. Buzzard <jonathan@buzzard.org.uk>\n"
 
-#define USAGE \
-"Usage: alarm [-l] [-h] [-v] time\n\
+#define USAGE_STRING "\
+Usage: alarm [-d] [-l] [-h] [-v] time\n\
 Set the alarm power on time or date of a Toshiba laptop\n\n\
+  -d,--disable       disable the alarm power on facility\n\
   -l,--localtime     hardware clock is set to localtime\n\
   -h,--help          display this help message\n\
   -v,--version       display version\n\
 Report bugs to jonathan@buzzard.org.uk\n"
 
 
-int support;
 
 enum {
 	ALARM_TIME  = 0x0001,
@@ -65,12 +61,45 @@ enum {
 };
 
 
+/*
+ * check the alarm features of the laptop
+ */
+int AlarmSupport(void)
+{
+	SciRegisters reg;
+	int support;
+
+	support = 0x0000;
+	SciOpenInterface();
+
+	reg.setting = SCI_ALARM_TIME;
+	if (SciGet(&reg)==SCI_SUCCESS) {
+		support |= ALARM_TIME;
+	}
+
+#define SCI_ALARM_EVERYDAY 0x0001
+
+	reg.setting = SCI_ALARM_DATE;
+	if (SciGet(&reg)==SCI_SUCCESS) {
+		support |= ALARM_DATE;
+		if ((reg.possible & SCI_ALARM_EVERYDAY)==0x0001)
+			support |= ALARM_EVERY;
+		if (SCI_YEAR(reg.possible)!=1990)
+			support |= ALARM_YEAR;
+	}
+
+	SciCloseInterface();
+	
+	return support;
+}
+
+
 int main(int argc, char *argv[])
 {
 	SciRegisters reg;
 	time_t timer;
-	int i,version,local;
-	int hour,minute;
+	int i,version,local,disable,support;
+	char buffer[64];
 	struct tm *alarm;
 
 	/* get the necessary I/O permissions */
@@ -92,50 +121,52 @@ int main(int argc, char *argv[])
 		return 1;
 	}
 
-	/* check the alarm features of the laptop */
+	/* check this model supports alarm power on */
 
-	support = 0x0000;
-	SciOpenInterface();
-
-	reg.setting = SCI_ALARM_TIME;
-	if (SciGet(&reg)==SCI_SUCCESS) {
-		support &= ALARM_TIME;
+	support = AlarmSupport();
+	if (support==0) {
+		fprintf(stderr, "alarm: power on not supported on "
+			"this model.\n");
+		return 1;
 	}
 
-#define SCI_ALARM_EVERYDAY 0x0001
-
-	reg.setting = SCI_ALARM_DATE;
-	if (SciGet(&reg)==SCI_SUCCESS) {
-		support &= ALARM_DATE;
-		if ((reg.possible & SCI_ALARM_EVERYDAY)==0x0001)
-			support &= ALARM_EVERY;
-		if (SCI_YEAR(reg.possible)!=1990)
-			support &= ALARM_YEAR;
-	}
-
-
-	SciCloseInterface();
-
 	/* process command line arguments */
 
 	local = 0;
+	disable = 0;
 	for (i=1;i<argc;i++) {
 		if ((!strcmp(argv[i], "-h")) || (!strcmp(argv[i], "--help"))) {
-			printf(USAGE);
+			printf(USAGE_STRING);
 			exit(0);
 		}
 		if ((!strcmp(argv[i], "-v")) || (!strcmp(argv[i], "--version"))) {
-			printf(VERSION);
+			printf(VERSION_STRING);
 			exit(0);
 		}
 		if ((!strcmp(argv[i], "-l")) || (!strcmp(argv[i], "--localtime"))) {
 			local = 1;
 		}		
+		if ((!strcmp(argv[i], "-d")) || (!strcmp(argv[i], "--disable"))) {
+			disable = 1;
+		}		
 	}
 
+	/* disable the alarm power on option if requested and exit */
+
+	if (disable==1) {
+		SciOpenInterface();
+		reg.setting = SCI_ALARM_TIME;
+		reg.current = SCI_ALARM_DISABLED;
+		SciSet(&reg);
+		SciCloseInterface();
+		printf("alarm: alarm power on disabled\n");
+		return 0;
+	}
+
+	/* parse the time string under */
+
 	if (local==1)
 		argv++;
-
 	optind = 1;
 	if (argc>=optind+1) {
 		timer = parsetime(i-optind, argv+optind);
@@ -157,19 +188,9 @@ int main(int argc, char *argv[])
 	else
 		alarm = gmtime(&timer);
 
-	printf("ALARM TIME STRING = %s", asctime(alarm));
-	printf("ALARM TIME %02d:%02d\n", alarm->tm_hour, alarm->tm_min);
-	printf("TIMER = %d\n",(int) timer);
-/*	printf("ALARM TIME 0x%04x\n", SCI_TIME(alarm->tm_hour, alarm->tm_min));
-*/
-	hour = alarm->tm_hour;
-	minute = alarm->tm_min;
-	printf("ALARM TIME %02d:%02d\n", hour, minute);
-
-
 	/* set the alarm time */
 
-	if (((support&ALARM_TIME)==0) && (timer!=0)) {
+	if (((support & ALARM_TIME)!=0) && (timer!=0)) {
 		SciOpenInterface();
 		reg.setting = SCI_ALARM_TIME;
 		reg.current = SCI_ALARM_ENABLED | SCI_TIME(alarm->tm_hour,
@@ -178,40 +199,61 @@ int main(int argc, char *argv[])
 		SciCloseInterface();
 	}
 
+	/* set the alarm date if supported */
+
+	if (((support & ALARM_DATE)!=0) && (timer!=0)) {
+		SciOpenInterface();
+		reg.setting = SCI_ALARM_DATE;
+		if ((support & ALARM_YEAR)!=0)
+			reg.current = SCI_FULLDATE((alarm->tm_year-90),
+				(1+alarm->tm_mon), alarm->tm_mday);
+		else
+			reg.current = SCI_DATE((1+alarm->tm_mon),
+				alarm->tm_mday);
+		i = SciSet(&reg);
+		SciCloseInterface();
+	}
+
+
 	/* display the current alarm power on setting */
 
+	alarm->tm_sec = 0;
 	SciOpenInterface();
 	reg.setting = SCI_ALARM_TIME;
 	if (SciGet(&reg)==SCI_SUCCESS) {
-		if (SCI_TIME_ON(reg.current)==SCI_ALARM_ENABLED)
-			printf("alarm power on set to %02d:%02d\n",
-				SCI_HOUR(reg.current), SCI_MINUTE(reg.current));
-		else
-			printf("alarm power on disabled\n");
-	} else {
-		fprintf(stderr, "alarm: power on time not supported on "
-			"this model.\n");
-	};
-	
-	reg.setting = SCI_ALARM_DATE;
-	if (SciGet(&reg)==SCI_SUCCESS) {
-		if ((reg.possible & 0x0001)==0x0001) {
-			printf("DATE: everyday\n");
+		if (SCI_TIME_ON(reg.current)==SCI_ALARM_ENABLED) {
+			alarm->tm_min = SCI_MINUTE(reg.current);
+			alarm->tm_hour = SCI_HOUR(reg.current);
 		} else {
-			printf("ax=0x%04x  bx=0x%04x  cx=0x%04x  dx=0x%04x\n",
-				reg.attribute, reg.setting, reg.current,
-				reg.possible);
+			printf("alarm power on disabled\n");
+			SciCloseInterface();
+			return 0;
 		}
 	}
 
-	
-	printf("year=%d month=%d day=%d\n", SCI_YEAR(reg.current),
-		SCI_MONTH(reg.current), SCI_DAY(reg.current));
-	
-	
-	
+	reg.setting = SCI_ALARM_DATE;
+	if (SciGet(&reg)==SCI_SUCCESS) {
+		alarm->tm_mday = SCI_DAY(reg.current);
+		alarm->tm_mon = (SCI_MONTH(reg.current))-1;
+		alarm->tm_year = SCI_YEAR(reg.current);
+		if ((reg.current & SCI_ALARM_EVERYDAY)==0x0001)
+			strftime(buffer, 64, "%R everyday", alarm);
+		if (alarm->tm_year==1990) {
+			alarm->tm_year = 1999-1900;
+			mktime(alarm);
+			strftime(buffer, 64, "%a %b %e %R %Y", alarm);
+		} else {
+			alarm->tm_year -= 1900;
+			mktime(alarm);
+			strftime(buffer, 64, "%a %b %e %R %Y", alarm);
+		}
+	} else {
+		strftime(buffer, 64, "%R", alarm);
+	}
+
 	SciCloseInterface();
 
+	printf("alarm power on set to %s\n", buffer);
 
 	return 0;
 }
